import asyncio
from argparse import ArgumentParser, ArgumentTypeError
from urllib.parse import urlparse

from aiohttp_client_cache import CacheBackend, CachedSession

from lparchive2epub.lib import lparchive2epub


def is_lparchive_url(arg):
    url = urlparse(arg)
    if not "lparchive.org" in url.geturl():
        raise ArgumentTypeError(f"{arg} is not an lparchive.org URL")
    if not url.path or url.path == "/":
        raise ArgumentTypeError(f"{arg} is not a let's play archive")
    return arg


arg_parser = ArgumentParser(
    prog="lparchive2epub",
    description="A tool to transform a Let's Play from lparchive.org to epub format."
)

arg_parser.add_argument("url", metavar="URL", nargs=1, type=is_lparchive_url)
arg_parser.add_argument("output", metavar="OUTPUT_FILE", nargs=1, type=str)
arg_parser.add_argument("--no-cache", action="store_true", help="Don't cache url calls")

async def amain(args):
    if args.no_cache:
        cache = None
    else:
        cache = CachedSession(cache=CacheBackend())
    await lparchive2epub(args.url[0], args.output[0], cache)

def main():
    args = arg_parser.parse_args()
    asyncio.run(amain(args))


if __name__ == '__main__':
    main()



import asyncio
import functools
from dataclasses import dataclass
from hashlib import blake2b
from typing import List, Tuple
import re
import aiohttp
from bs4 import BeautifulSoup
from ebooklib import epub
from ebooklib.epub import EpubHtml, EpubImage, EpubBook
from tqdm.asyncio import tqdm
import datetime
from urllib.parse import urljoin, urlparse, urlunparse

from lparchive2epub.style import get_style_item

def get_blake2b_hash(content: bytes) -> str:
    hasher = blake2b()
    hasher.update(content)
    return hasher.hexdigest()

@dataclass(order=True)
class Chapters:
    num: int
    original_href: str
    txt: BeautifulSoup
    new_href: str
    original_href_slug: str

    def __hash__(self):
        return hash((self.original_href, str(self.txt.text)))

    def __post_init__(self):
        self.sort_index = self.num


@dataclass(order=True)
class Image:
    num: int
    src: str
    media_type: str
    tag: BeautifulSoup
    url: str

    def __post_init__(self):
        self.sort_index = self.num


@dataclass(order=True)
class IndexedEpubImage:
    num: int
    hash: str
    old_name: str
    new_name: str
    data: EpubImage

    def __post_init__(self):
        self.sort_index = self.num


@dataclass
class Intro:
    title: str
    language: str
    author: str
    intro: BeautifulSoup
    images: List[Image]
    chapters: List[Chapters]
    published_on: str


@dataclass
class Update:
    content: BeautifulSoup
    images: List[Image]


LP_SPECIFIC_NAMES = {
    "Dwarf-Fortress-Boatmurdered": ["Introduction"],
    "Arcanum": ["Intro", "Epilogue"],
    "Martian-Dreams": ["Cast"]
}

FIND_FILES = re.compile(r"([.])[^/.]+$")
FIND_ANCHOR_LINKS = re.compile(r"(#)[^/.]+$")


class Extractor:

    @staticmethod
    def fix_links(content, chapters, current_url: str):
        if not current_url.endswith("/"):
            current_url += "/"

        all_links = [x for x in content.find_all("a") if x.get("href", None)]
        for link in all_links:
            link["href"] = link["href"].replace("../", "")

        local_assets = (x for x in all_links if FIND_FILES.search(x.get("href")) and "/" not in x.get("href"))

        for link_to_asset in local_assets:
            link_to_asset["href"] = current_url + link_to_asset["href"]

        for c in chapters:
            links = content.find_all("a", href=re.compile(c.original_href_slug))
            to_update = (x for x in links if not FIND_FILES.search(x.get("href")))
            for link_to_update in to_update:
                link_to_update["href"] = link_to_update["href"].replace(c.original_href_slug, c.new_href + "/")
            asset_links = (x for x in links if FIND_FILES.search(x.get("href")))
            for link_to_asset in asset_links:
                link_to_asset["href"] = link_to_asset["href"].replace(c.original_href_slug, c.original_href + "/")

    @staticmethod
    def get_known_update_names(url: str) -> List[str]:
        knownUpdateNames = ["Update"]
        extension = next((x for x in LP_SPECIFIC_NAMES.keys() if x in url), None)
        if extension:
            knownUpdateNames.extend(LP_SPECIFIC_NAMES[extension])
        return knownUpdateNames

    @staticmethod
    def intro(url: str, p: BeautifulSoup) -> Intro:
        title = p.title.text
        author = next(x for x in p.find_all("meta") if x.get("name", None) == "author").get("content", None)
        language = "en"
        content = p.find("div", id="content")
        chapters = Extractor.all_chapters(url, p)
        Extractor.fix_links(content, chapters, url)

        images = Extractor.all_images(content, url)

        archival = p.find("li", id="archival")
        published_on = archival.find("strong").text

        return Intro(
            title=title, language=language, author=author, intro=content, images=images, chapters=chapters, published_on=published_on
        )

    @staticmethod
    def all_chapters(root_url: str, p: BeautifulSoup) -> List[Chapters]:
        content = p.find("div", id="content")

        chapters = content.find_all("a")

        known_update_names = Extractor.get_known_update_names(root_url)
        chapters = (x for x in chapters if any(a in x.get("href", "") for a in known_update_names))
        chapters = (x for x in chapters if "lparchive.org" not in x.get("href", ""))
        chapters = (x for x in chapters if not FIND_FILES.search(x.get("href", "")))
        chapters = (x for x in chapters if not FIND_ANCHOR_LINKS.search(x.get("href", "")))

        chapters = list(chapters)

        if not chapters:
            return []

        def build_chapter(chap: Tuple[int, BeautifulSoup]) -> Chapters:
            i, c = chap
            original_href_slug = c.get("href", None)
            original_href = f"{root_url}/{original_href_slug}"
            if not original_href.endswith("/"):
                original_href += "/"
            return Chapters(
                num=i,
                original_href=original_href,
                txt=c.text,
                new_href=f"update_{i}.xhtml",
                original_href_slug=original_href_slug,
            )

        chapters = list(map(build_chapter, enumerate(chapters)))

        return chapters


    @staticmethod
    def all_images(content: BeautifulSoup, root_url: str) -> List[Image]:
        # Get direct image tags
        images = content.find_all("img")
        images = (x for x in images if x.get("src", None) is not None)
        r = []

        root = str(root_url)
        if root.endswith("/"):
            root = root[:-1]

        def get_media_type(base : str) -> str:
            if "jpg" in base:
                base = base.replace("jpg", "jpeg")
            return base
        
        def remove_extra_dots(url):
            parsed = list(urlparse(url))
            dirs = []
            for name in parsed[2].split("/"):
                if name == "..":
                    if len(dirs) > 1:
                        dirs.pop()
                else:
                    dirs.append(name)
            parsed[2] = "/".join(dirs)
            return urlunparse(parsed)

        def get_full_url(src: str) -> str:
            if root_url in src:
                full = src
            elif "http://" in src or "https://" in src:
                full = src
            else:
                full = f"{root}/{src}"
            if ".." in full:
                full = remove_extra_dots(full)
            return full

        # Add direct images
        for i, x in enumerate(images):
            r.append(Image(
                num=i,
                src=x['src'],
                media_type=get_media_type(x["src"][-3:]),
                tag=x,
                url=get_full_url(x["src"])
            ))

        # Find image links - typically "Click here for full image" links
        image_links = content.find_all("a", href=lambda x: x and (
                    x.endswith('.jpg') or x.endswith('.png') or x.endswith('.gif')))

        # Add linked images to be saved, but don't modify the HTML structure
        for i, link in enumerate(image_links, start=len(r)):
            r.append(Image(
                num=i,
                src=link['href'],
                media_type=get_media_type(link['href'][-3:]),
                tag=link,  # Use the link tag itself
                url=get_full_url(link['href'])
            ))

        return r

    @staticmethod
    def get_update(chapters: List[Chapters], p: BeautifulSoup, chapter: Chapters) -> Update:
        content = p.find("div", id="content")
        images = Extractor.all_images(content, chapter.original_href)
        Extractor.fix_links(content, chapters, chapter.original_href)
        return Update(content=content, images=images)


@dataclass(order=True)
class Page:
    num: int
    chapter: EpubHtml
    images: List[IndexedEpubImage]

    def __post_init__(self):
        self.sort_index = self.num


async def _get_image(session: aiohttp.ClientSession, img: Image) -> IndexedEpubImage:
    # Get the image URL from either src (for img tags) or href (for a tags)
    img_url = img.url
    async with session.get(img_url) as r:
        if r.status != 200:
            raise RuntimeError(f"Failed to get image {img_url}, img_url: {img_url}, status code: {r.status}")
        media_type = img.media_type
        content = await r.content.read()
        img_hash = get_blake2b_hash(content)
        new_name = f"images/{img_hash}.{media_type}"
        return IndexedEpubImage(
            img.num,
            img_hash,
            img_url,  # Use the original URL as old_name
            new_name,
            EpubImage(
                uid=f"i{img_hash}",
                file_name=new_name,
                media_type=f"image/{media_type}",
                content=content)
        )


async def build_intro(session: aiohttp.ClientSession, url_root: str, intro: Intro) -> Page:
    intro_chapter = epub.EpubHtml(title="Introduction", file_name="introduction.xhtml", lang=intro.language)
    intro_chapter.add_item(get_style_item())
    tasks = []
    for image in intro.images:
        task = asyncio.ensure_future(_get_image(session, image))
        tasks.append(task)

    images = await asyncio.gather(*tasks)

    for image in images:
        intro.intro = replace_img_name(intro.intro, image)

    intro_chapter.content = str(intro.intro)

    return Page(0, intro_chapter, images)


def replace_img_name(content: BeautifulSoup, image: IndexedEpubImage) -> BeautifulSoup:
    # Handle img tags
    to_change_imgs = (x for x in content.find_all("img") if x["src"] == image.old_name)
    for item in to_change_imgs:
        item["src"] = image.new_name

    # Handle linked images
    to_change_links = (x for x in content.find_all("a") if x.get("href", "") == image.old_name)
    for item in to_change_links:
        item["href"] = image.new_name

    return content

def batch(iterable, n=1):
    l = len(iterable)
    for ndx in range(0, l, n):
        yield iterable[ndx:min(ndx + n, l)]

async def build_update(session: aiohttp.ClientSession, chapter: Chapters, data: Update, intro: Intro) -> Page:
    update_chapter = epub.EpubHtml(title=str(chapter.txt), file_name=chapter.new_href,
                                   lang=intro.language)  # TODO: fix language
    update_chapter.add_item(get_style_item())

    img_builder = functools.partial(_get_image, session)

    images = []
    
    for chunk in batch(data.images, 10):
        tasks = []
        for image in chunk:
            task = asyncio.ensure_future(img_builder(image))
            tasks.append(task)

        images.extend(await asyncio.gather(*tasks))


    for image in images:
        data.content = replace_img_name(data.content, image)

    update_chapter.content = str(data.content)

    return Page(chapter.num, update_chapter, images)


def add_page(known_images: dict, book: EpubBook, toc: List, spine: List, page: Page):
    book.add_item(page.chapter)
    for img in page.images:
        if img.hash not in known_images["hashes"]:
            book.add_item(img.data)
            known_images["hashes"].append(img.hash)
    toc.append(epub.Link(page.chapter.file_name, page.chapter.title, page.chapter.id))
    spine.append(page.chapter)


async def build_single_page(session: aiohttp.ClientSession, intro: Intro, chapter: Chapters, all_chapters: List[Chapters], pbar) -> Page:
    chapter_page = await session.get(chapter.original_href)
    chapter_bs = get_cleaned_html(await chapter_page.text())
    update = Extractor.get_update(all_chapters, chapter_bs, chapter)
    r = await build_update(session, chapter, update, intro)
    pbar.update(1)
    return r


class DummyUpdater:

    def __init__(self, *args, **kwargs):
        pass

    #todo: everything


async def lparchive2epub(url: str, file: str, root_session: aiohttp.ClientSession | None = None, writer=tqdm.write):
    if url.endswith("/"):
        url = url[:-1]
    if root_session is None:
        root_session = aiohttp.ClientSession()
    async with root_session as session:
        await do(url, file, session, writer)


tags_to_clean = [
    "st1:place",
    "st1:city",
    "st1:country-region",
    "st1:placename",
    "st1:placetype",
    "st1:state",
    "o:p"
]


def get_cleaned_html(page: str) -> BeautifulSoup:
    xml_version = BeautifulSoup(page, "html.parser")
    for tag in tags_to_clean:
        while found := xml_version.find(tag):
            found.unwrap()
    return BeautifulSoup(str(xml_version), "html.parser")


async def do(url: str, file: str, session: aiohttp.ClientSession, writer):
    writer(f"extracting lp from {url}")
    writer("getting landing page")
    page = await session.get(url)

    landing = get_cleaned_html(await page.text())

    book = epub.EpubBook()
    intro = Extractor.intro(url, landing)

    toc = []
    spine = ["nav"]

    writer("building intro")
    epub_intro = await build_intro(session, url, intro)

    known_images = {
        "hashes": [],
    }

    add_page(known_images, book, toc, spine, epub_intro)

    # pages takes a vastly inequal times to get, so it's faster to gather and sort
    # gotta go fast, but also keep the page order.

    tasks = []

    writer("building chapters / updates")
    

    pages = []

    with tqdm(total=len(intro.chapters), desc="Gathering and building pages") as pbar:
        for chunk in batch(intro.chapters, 10):
            for chapter in chunk:
                task = asyncio.ensure_future(build_single_page(session, intro, chapter, intro.chapters, pbar))
                tasks.append(task)

        pages.extend(await asyncio.gather(*tasks))

    pages = sorted(pages)

    for p in tqdm(pages, desc="Adding pages to book"):
        add_page(known_images, book, toc, spine, p)

    writer("preparing book structure")
    book.toc = toc

    book.add_item(epub.EpubNcx())
    book.add_item(epub.EpubNav())

    book.spine = spine

    book.add_item(get_style_item())

    book.add_author(intro.author)
    book.set_title(intro.title)
    book.set_language(intro.language)
    book.add_metadata("DC", "source", url)
    book.add_metadata("DC", "identifier", f"lparchive2epub-{hash(intro.title)}-{hash(intro.author)}-{hash(url)}")

    book.set_identifier(f"lparchive2epub-{get_blake2b_hash(intro.title.encode("utf-8"))}-{get_blake2b_hash(intro.author.encode("utf-8"))}-{get_blake2b_hash(url.encode("utf-8"))}")

    mtime = datetime.datetime.strptime(intro.published_on, "%b %d, %Y")

    writer("Writing book file")
    epub.write_epub(file, book, options = {"mtime": mtime})



# Courtesy of https://github.com/mattharrison/epub-css-starter-kit

from ebooklib import epub


def get_style_item():
    return epub.EpubItem(uid="style_nav",
                        file_name="style/nav.css",
                        media_type="text/css",
                        content=style)

style = """/* This assumes geometric header shrinkage */
/* Also, it tries to make h2 be 1em */
html, body, div, span, applet, object, iframe, h1, h2, h3, h4, h5, h6, p, blockquote, pre, a, abbr, acronym, address, big, cite, code, del, dfn, em, img, ins, kbd, q, s, samp, small, strike, strong, sub, sup, tt, var, b, u, i, center, dl, dt, dd, fieldset, form, label, legend, table, caption, tbody, tfoot, thead, tr, th, td, article, aside, canvas, details, embed, figure, figcaption, footer, header, hgroup, menu, nav, output, ruby, section, summary, time, mark, audio, video {
  /* Note kindle hates margin:0 ! (or margin-left or margin-top set) it inserts newlines galore */
  /* margin: 0 */
  margin-right: 0;
  padding: 0;
  border: 0;
  font-size: 100%;
  /* font: inherit */
  vertical-align: baseline; }

/* optimal sizing see http://demosthenes.info/blog/578/Crafting-Optimal-Type-Sizes-For-Web-Pages */
/* kobo and nook dislike this */
/*html */
/*  font-size: 62.5% */
/*body */
/*  font-size: 1.6rem */
/*  line-height: 2.56rem */
/*  text-rendering: optimizeLegibility */
table {
  border-collapse: collapse;
  border-spacing: 0; }

/* end reset */
@page {
  margin-top: 30px;
  margin-bottom: 20px; }

div.cover {
  text-align: center;
  page-break-after: always;
  padding: 0px;
  margin: 0px; }
  div.cover img {
    height: 100%;
    max-width: 100%;
    padding: 10px;
    margin: 0px;
    background-color: #cccccc; }

.half {
  max-width: 50%; }

.tenth {
  max-width: 10%;
  width: 10%; }

.cover-img {
  height: 100%;
  max-width: 100%;
  padding: 0px;
  margin: 0px; }

/* font plan- serif text, sans headers */
h1, h2, h3, h4, h5, h6 {
  hyphens: none !important;
  -moz-hyphens: none !important;
  -webkit-hyphens: none !important;
  adobe-hyphenate: none !important;
  page-break-after: avoid;
  page-break-inside: avoid;
  text-indent: 0px;
  text-align: left;
  font-family: Helvetica, Arial, sans-serif; }

h1 {
  font-size: 1.6em;
  margin-bottom: 3.2em; }

.title h1 {
  margin-bottom: 0px;
  margin-top: 3.2em; }

h2 {
  font-size: 1em;
  margin-top: 0.5em;
  margin-bottom: 0.5em; }

h3 {
  font-size: 0.625em; }

h4 {
  font-size: 0.391em; }

h5 {
  font-size: 0.244em; }

h6 {
  font-size: 0.153em; }

/* Do not indent first paragraph. Mobi will need class='first-para' */
h1 + p, h2 + p, h3 + p, h4 + p, h5 + p, h6 + p {
  text-indent: 0; }

p {
  /* paperwhite defaults to sans */
  font-family: "Palatino", "Times New Roman", Caecilia, serif;
  -webkit-hyphens: auto;
  -moz-hyphens: auto;
  hyphens: auto;
  hyphenate-after: 3;
  hyphenate-before: 3;
  hyphenate-lines: 2;
  -webkit-hyphenate-after: 3;
  -webkit-hyphenate-before: 3;
  -webkit-hyphenate-lines: 2;
  line-height: 1.5em;
  margin: 0;
  text-align: justify;
  text-indent: 1em;
  orphans: 2;
  widows: 2; }
  p.first-para, p.first-para-chapter, p.note-p-first {
    text-indent: 0; }
  p.first-para-chapter::first-line {
    /* handle run-in */
    font-variant: small-caps; }
  p.no-indent {
    text-indent: 0; }

.no-hyphens {
  hyphens: none !important;
  -moz-hyphens: none !important;
  -webkit-hyphens: none !important;
  adobe-hyphenate: none !important; }

.rtl {
  direction: rtl;
  float: right; }

.drop {
  overflow: hidden;
  line-height: 89%;
  height: 0.8em;
  font-size: 281%;
  margin-right: 0.075em;
  float: left; }

.dropcap {
  line-height: 100%;
  font-size: 341%;
  margin-right: 0.075em;
  margin-top: -0.22em;
  float: left;
  height: 0.8em; }

/* lists */
ul, ol, dl {
  margin: 1em 0 1em 0;
  text-align: left; }

li {
  font-family: "Palatino", "Times New Roman", Caecilia, serif;
  line-height: 1.5em;
  orphans: 2;
  widows: 2;
  text-align: justify;
  text-indent: 0;
  margin: 0; }
  li p {
    /* Fix paragraph indenting inside of lists */
    text-indent: 0em; }

dt {
  font-weight: bold;
  font-family: Helvetica, Arial, sans-serif; }

dd {
  line-height: 1.5em;
  font-family: "Palatino", "Times New Roman", Caecilia, serif; }
  dd p {
    /* Fix paragraph indenting inside of definition lists */
    text-indent: 0em; }

blockquote {
  margin-left: 1em;
  margin-right: 1em;
  line-height: 1.5em;
  font-style: italic; }
  blockquote p.first-para, blockquote p {
    text-indent: 0; }

pre, tt, code, samp, kbd {
  font-family: "Courier New", Courier, monospace;
  word-wrap: break-word; }

pre {
  font-size: 0.8em;
  line-height: 1.2em;
  margin-left: 1em;
  /* margin-top: 1em */
  margin-bottom: 1em;
  white-space: pre-wrap;
  display: block; }

img {
  border-radius: 0.3em;
  -webkit-border-radius: 0.3em;
  -webkit-box-shadow: rgba(0, 0, 0, 0.15) 0 1px 4px;
  box-shadow: rgba(0, 0, 0, 0.15) 0 1px 4px;
  box-sizing: border-box;
  border: white 0.5em solid;
  /* Don't go too big on images, let reader zoom in if they care to */
  max-width: 80%;
  max-height: 80%; }

img.pwhack {
  /* Paperwhite hack */
  width: 100%; }

.group {
  page-break-inside: avoid; }

.caption {
  text-align: center;
  font-size: 0.8em;
  font-weight: bold; }

p img {
  border-radius: 0;
  border: none; }

figure {
  /* These first 3 should center figures */
  padding: 1em;
  background-color: #cccccc;
  border: 1px solid black;
  text-align: center; }
  figure figcaption {
    text-align: center;
    font-size: 0.8em;
    font-weight: bold; }

div.div-literal-block-admonition {
  margin-left: 1em;
  background-color: #cccccc; }
div.note, div.tip, div.hint {
  margin: 1em 0 1em 0 !important;
  background-color: #cccccc;
  padding: 1em !important;
  /* kindle is finnicky with borders, bottoms dissappear, width is ignored */
  border-top: 0px solid #cccccc;
  border-bottom: 0px dashed #cccccc;
  page-break-inside: avoid; }

/* sidebar */
p.note-title, .admonition-title {
  margin-top: 0;
  /*mobi doesn't like div margins */
  font-variant: small-caps;
  font-size: 0.9em;
  text-align: center;
  font-weight: bold;
  font-style: normal;
  -webkit-hyphens: none;
  -moz-hyphens: none;
  hyphens: none;
  /* margin:0 1em 0 1em */ }

div.note p, .note-p {
  text-indent: 1em;
  margin-left: 0;
  margin-right: 0; }

/*  font-style: italic */
/* Since Kindle doesn't like multiple classes have to have combinations */
div.note p.note-p-first {
  text-indent: 0;
  margin-left: 0;
  margin-right: 0; }

/* Tables */
table {
  /*width: 100% */
  page-break-inside: avoid;
  border: 1px;
  /* centers on kf8 */
  margin: 1em auto;
  border-collapse: collapse;
  border-spacing: 0; }

th {
  font-variant: small-caps;
  padding: 5px !important;
  vertical-align: baseline;
  border-bottom: 1px solid black; }

td {
  font-family: "Palatino", "Times New Roman", Caecilia, serif;
  font-size: small;
  hyphens: none;
  -moz-hyphens: none;
  -webkit-hyphens: none;
  padding: 5px !important;
  page-break-inside: avoid;
  text-align: left;
  text-indent: 0;
  vertical-align: baseline; }

td:nth-last-child {
  border-bottom: 1px solid black; }

.zebra {
  /* shade background by groups of three */ }
  .zebra tr th {
    background-color: white; }
  .zebra tr:nth-child(6n-1), .zebra tr:nth-child(6n+0), .zebra tr:nth-child(6n+1) {
    background-color: #cccccc; }

sup {
  vertical-align: super;
  font-size: 0.5em;
  line-height: 0.5em; }

sub {
  vertical-align: sub;
  font-size: 0.5em;
  line-height: 0.5em; }

table.footnote {
  margin: 0.5em 0em 0em 0em; }

.footnote {
  font-size: 0.8em; }

.footnote-link {
  font-size: 0.8em;
  vertical-align: super; }

.tocEntry-1 a {
  /* empty */
  font-weight: bold;
  text-decoration: none;
  color: black; }

.tocEntry-2 a {
  margin-left: 1em;
  text-indent: 1em;
  text-decoration: none;
  color: black; }

.tocEntry-3 a {
  text-indent: 2em;
  text-decoration: none;
  color: black; }

.tocEntry-4 a {
  text-indent: 3em;
  text-decoration: none;
  color: black; }

.copyright-top {
  margin-top: 6em; }

.page-break-before {
  page-break-before: always; }

.page-break-after {
  page-break-after: always; }

.center {
  text-indent: 0;
  text-align: center;
  margin-left: auto;
  margin-right: auto;
  display: block; }

.right {
  text-align: right; }

.left {
  text-align: left; }

.f-right {
  float: right; }

.f-left {
  float: left; }

/* Samples */
.ingredient {
  page-break-inside: avoid; }

.box-example {
  background-color: #8ae234;
  margin: 2em;
  padding: 1em;
  border: 2px dashed #ef2929; }

.blue {
  background-color: blue; }

.dashed {
  border: 2px dashed #ef2929; }

.padding-only {
  padding: 1em; }

.margin-only {
  margin: 2em; }

.smaller {
  font-size: 0.8em; }

.em1 {
  font-size: 0.5em; }

.em2 {
  font-size: 0.75em; }

.em3 {
  font-size: 1em; }

.em4 {
  font-size: 1.5em; }

.em5 {
  font-size: 2em; }

.per1 {
  font-size: 50%; }

.per2 {
  font-size: 75%; }

.per3 {
  font-size: 100%; }

.per4 {
  font-size: 150%; }

.per5 {
  font-size: 200%; }

.mousepoem p {
  line-height: 0;
  margin-left: 1em; }

.per100 {
  font-size: 100%;
  line-height: 0.9em; }

.per90 {
  font-size: 90%;
  line-height: 0.9em; }

.per80 {
  font-size: 80%;
  line-height: 0.9em; }

.per70 {
  font-size: 70%;
  line-height: 0.9em; }

.per60 {
  font-size: 60%;
  line-height: 0.9em; }

.per50 {
  font-size: 50%;
  line-height: 1.05em; }

.per40 {
  font-size: 40%;
  line-height: 0.9em; }

.size1 {
  font-size: x-small; }

.size2 {
  font-size: small; }

.size3 {
  /* default */
  font-size: medium; }

.size4 {
  font-size: large; }

.size5 {
  font-size: x-large; }

/* Poetic margins */
.stanza {
  margin-top: 1em;
  font-family: serif;
  padding-left: 1em; }
  .stanza p {
    padding-left: 1em; }

.poetry {
  margin: 1em; }

/*line number */
.ln {
  float: left;
  color: #999999;
  font-size: 0.8em;
  font-style: italic; }

.pos1 {
  margin-left: 1em;
  text-indent: -1em; }

.pos2 {
  margin-left: 2em;
  text-indent: -1em; }

.pos3 {
  margin-left: 3em;
  text-indent: -1em; }

.pos4 {
  margin-left: 4em;
  text-indent: -1em; }

@font-face {
  font-family: Inconsolata Mono;
  font-style: normal;
  font-weight: normal;
  src: url("Inconsolata.otf"); }

.normal-mono {
  font-family: "Courier New", Courier, monospace; }

tt, pre, .mono {
  /* Kindle Keyboard has KF8 but no font support, fallback to default mono */
  font-family: "Inconsolata Mono", "Courier New", Courier, monospace;
  font-style: normal; }

@font-face {
  font-family: mgopen modata;
  font-style: normal;
  font-weight: normal;
  font-size: 0.5em;
  src: url("MgOpenModataRegular.ttf"); }

.modata {
  font-family: "mgopen modata"; }

@font-face {
  font-family: hidden;
  font-style: normal;
  font-weight: normal;
  font-size: 1em;
  src: url("invisible1.ttf"); }

.hidden-font {
  font-family: "hidden"; }

/* Nook works to here :) */
/* media queries at bottom to not confuse other platforms */
@media (min-width: 200px) {
  .px200 {
    color: #8ae234; } }
@media (min-width: 400px) {
  .px400 {
    color: #8ae234; } }
@media (min-width: 800px) {
  .px800 {
    color: #8ae234; } }
@media (min-width: 1200px) {
  .px1200 {
    color: #8ae234; } }
/* broke nook! */
/*/* WIP device specific... */
/*@media (min-width: 600px) and (height: 800px) and (amzn-kf8) */
/*  /* Kindle Keyboard w/ KF8 */
/*  .kk */
/*    color: $green */

/*/* @media (min-width: 768px) and (height: 1024px) and (amzn-kf8) */
/*@media (min-width: 748px) and (min-height: 1004px) and (amzn-kf8) */
/*  /* Kindle Paperwhite */
/*  .kpw */
/*    color: $green */
/*@media (width: 600px) and (height: 1024px) and (amzn-kf8) */
/*  /* Kindle Fire */
/*  .kf */
/*    color: $green */
/*/* Retina iPad */
/*@media (-webkit-min-device-pixel-ratio: 1.5), (min-device-pixel-ratio: 1.5) */
/*  .retina */
/*    color: $green */
@media amzn-kf8 {
  span.dropcapold {
    font-size: 300%;
    font-weight: bold;
    height: 1em;
    float: left;
    margin: -0.2em 0.1em 0 0.1em; }

  .dropcap {
    line-height: 100%;
    font-size: 341%;
    margin-right: 0.075em;
    margin-top: -0.22em;
    float: left;
    height: 0.8em; } }
@media amzn-mobi {
  span.dropcap {
    font-size: 1.5em;
    font-weight: bold; }

  /*  tt, pre */
  /*    font-size: 3 */
  /*     Size table */
  /* name     becomes */
  /* x-small  2 */
  /* small    3 */
  /* medium   4 */
  /*     1em  default (nothing) */
  tt {
    /* mobi fun */
    /* font-size: x-small  /* turns into <font size="2" */
    font-family: "Courier New", Courier, monospace; }

  pre {
    margin-left: 1em;
    margin-bottom: 1em;
    /* mobi fun */
    font-size: x-small;
    font-family: "Courier New", Courier, monospace;
    white-space: pre-wrap;
    display: block; }
    pre .no-indent {
      margin-left: 0em;
      text-indent: 0em; }

  div.no-indent {
    margin-left: 0em;
    text-indent: 0em; }

  /* Sass wants to add em to the end..., hardcoded for now */
  h1 {
    font-size: 2em; }

  h2 {
    font-size: 1em; }

  h3 {
    font-size: 2em; }

  h4 {
    font-size: 1em; }

  blockquote {
    /* something in this css causes blockquotes to get doubly indented! (BUG) */
    font-style: italics;
    margin-left: 0em;
    margin-right: 0em; }

  /* descendant selectors don't work in mobi7 infact this will override the preview h1 defintion! */
  /* h1 tt, h2 tt { */
  /*   font-size: 1em; */
  /*   color: green;} */
  div.note {
    border: 1px solid black;
    /*text-indent: 1em */ }

  div.note, .note-p {
    text-indent: 1em;
    margin-left: 0;
    margin-right: 0;
    font-style: italic; }

  /* Since Kindle doesn't like multiple classes have to have combinations (fixed in 2.7) */
  .note-p-first {
    text-indent: 0;
    margin-left: 1em;
    margin-right: 1em; }

  .note-p {
    text-indent: 1em;
    margin-left: 1em;
    margin-right: 1em; }

  /* Poetry handing indent hacks */
  /* see http://ebookarchitects.com/blog/backwards-compatible-poetry-for-kf8mobi/ */
  /* and http://www.pigsgourdsandwikis.com/2012/01/media-queries-for-formatting-poetry-on.html */
  .pos1 {
    text-indent: -1em; }

  .pos2 {
    text-indent: -1em; }

  .pos3 {
    text-indent: -1em; }

  .pos4 {
    text-indent: -1em; } }
/* does nook ignore this? */
.green {
  color: #8ae234; }

/*These break NOOK! */
/*@media (monochrome) */
/*  .monochrome */
/*    color: $green */
/*@media (color) */
/*  .color */
/*    color: $green */
.linked-fullsize-image {
    max-width: 100%;
    height: auto;
    display: block;
    margin: 1em 0;
}
"""